import asyncio
import datetime
from datetime import datetime
from enum import Enum
from enum import IntEnum
from time import time
from time import localtime, strftime

from modules.lib.report import Report
from modules.lib.active_reporter import ActiveReporter
from modules.azure.report_repository import AzureReportRepository

import socket
import select
import pickle
buffer_size = 4092

class DataState(IntEnum):
    WAIT_DATA    = 1,
    READY_REPORT = 2 

class G4BasicReporter(ActiveReporter):

    def __init__(self, report_queue, report_repository, ip_address, port):
        super().__init__(report_queue, report_repository)
        self._curr_state     = DataState.WAIT_DATA
        self._rcvdata        = ''
        self._ip_address     = ip_address
        self._port           = port 

    def request_stop(self):
        super().request_stop()

    def _before_loop(self):
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.socket.bind((self._ip_address, self._port))
        self.socket.listen(5)

    def _after_loop(self):
        pass

    def _handle_state(self):
        if DataState.WAIT_DATA == self._curr_state:
            self._handle_wait_measurement()
        elif DataState.READY_REPORT == self._curr_state:
            self._handle_ready_report()

    def _do_transit_action(self, next_state):
        # print("transient to the state ", next_state)
        pass

    def _handle_wait_measurement(self):
        r, _, _ = select.select([self.socket.fileno()], [], [], 1)
        if len(r) != 0:
            conn, addr = self.socket.accept()
            data = conn.recv(buffer_size)
            self._rcvdata = pickle.loads(data) 
            print(self._rcvdata)
            conn.close()
            self._transit_state(DataState.READY_REPORT)
        # It should be add error handling

    def _handle_ready_report(self):
        # make report and enqueue it
        report = Report.report_now(
            'measurement',
            type=AzureReportRepository.BAG_OF_REPORT,
            key='',
            value=self._rcvdata,
            unit=''
        )
        self._report_queue.push(report)

        # update status
        self._transit_state(DataState.WAIT_DATA)

#
# End of File
#

